#include <mega8.h>
#include <delay.h>

// I2C Bus functions
#asm
   .equ __i2c_port=0x18 ;PORTB
   .equ __sda_bit=4
   .equ __scl_bit=2
#endasm
#include <i2c.h>

// DS1307 Real Time Clock functions
#include <ds1307.c>

unsigned char global_mode = 0;			//   
#define clock_mode	0                       //  
#define clock_setup	1                       //  
#define alarm_setup	2                       //  
#define date_view       3                       //  

#define led_on()	PORTC.5 = 1		//      -  
#define led_off()	PORTC.5 = 0
#define led_inv()	PORTC.5 = !PORTC.5

#define led_g_on()	PORTD.3 = 1
#define led_g_off()	PORTD.3 = 0
#define led_g_inv()	PORTD.3 = !PORTD.3

#define led_r_on()	PORTC.1 = 1
#define led_r_off()	PORTC.1 = 0
#define led_r_inv()	PORTC.1 = !PORTC.1

#define led_b_on()	PORTC.2 = 1
#define led_b_off()	PORTC.2 = 0
#define led_b_inv()	PORTC.2 = !PORTC.2

#define SPI_PORT        PORTB
#define SPI_DDR         DDRB

#define SS_PIN          2
#define MOSI_PIN        3
#define MISO_PIN        4
#define SCK_PIN         5 
                  
#define strobe_PORT     PORTD
#define strobe_DDR      DDRD
#define strobe_PIN      4
#define strobe_toggle() {strobe_PORT.strobe_PIN = 1; strobe_PORT.strobe_PIN = 0;}

#define button		!PINC.3			// 
unsigned int button_cnt = 0;                    //     

#ifndef SPIF
#define SPIF 7
#endif

void SPIInit(void)		//    SPI
        {
        unsigned char temp;     // 
        
                                // setup SPI I/O pins
        SPI_DDR.SCK_PIN = 1;    // set SCK as output
        SPI_PORT.SCK_PIN = 0;   // set SCK low
        
//         SPI_DDR.MISO_PIN = 0;   // set MISO as input
//         SPI_PORT.MISO_PIN = 1;  // set pullup
        
        SPI_DDR.MOSI_PIN = 1;   // set MOSI as output
        
//        SPI_DDR.SS_PIN = 1;     // SS must be output for Master mode to work
        
        strobe_DDR.strobe_PIN = 1;      //     
        strobe_PORT.strobe_PIN = 0;     //  
        
                                //SPI Mode: 0
	                                
        SPCR.0 = 0;             //SPI Clock Rate Select (fosc/4)
	SPCR.1 = 0;             //
	SPSR.0 = 1;             //SPI2X: Double SPI Speed (1) Bit, ..    fosc/2
	SPCR.2 = 0;             //CPHA: Clock Phase (  - 1)- . 
	SPCR.3 = 0;             //CPOL: Clock Polarity (0:   - ,   - ) - . 
	SPCR.4 = 1;             //MSTR: Master(1)/Slave(0) Select
	SPCR.5 = 0;             //DORD: Data Order (LSB first = 1; MSB first = 0)
	//SPCR.6 = 1;             //SPE: SPI Enable = 1
	SPCR.7 = 0;             //SPIE: SPI Interrupt Enable = 1
	
 	temp = SPSR;            //clear SPI status (. )
 	temp = SPDR;
	};

unsigned char byte1, byte2;

unsigned char hour, min, sec;                                   //  
unsigned char alarm_complete;                                   //   

//unsigned char num1, num2, num3, num4;

unsigned char null;                                             // null,    null ))

unsigned int timeout = 0;                                       //    

eeprom unsigned char null2, null3, a_hour = 1, a_min = 1, alarm_flag = 0;	//     

void sound_on (int time)	//  
        {
        int i = 0;
        while (i < time*2)
                {  
                PORTB.0 = 1;
                delay_us(250);
                PORTB.0 = 0;
                delay_us(250);                
                i++;
                }
        }

#define ADC_VREF_TYPE 0xC0

unsigned int volt = 0;

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
	{
	ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
	delay_us(10);
	ADCSRA|=0x40;
	while ((ADCSRA & 0x10)==0);
	ADCSRA|=0x10;
	return ADCW;
	}
	
void print_decode (unsigned char a, unsigned char b, unsigned char c, unsigned char d)
	{
	unsigned char temp1=0, temp2=0;
	
	switch (a)
		{
		case 0: temp1 = 0b00000111; break;
		case 1: temp1 = 0b00000101; break;
		case 2: temp1 = 0b00000100; break;
		case 3: temp1 = 0b00001001; break;
		case 4: temp1 = 0b00001000; break;
		case 5: temp1 = 0b00000010; break;
		case 6: temp1 = 0b00000011; break;
		case 7: temp1 = 0b00000001; break;
		case 8: temp1 = 0b00000000; break;
		case 9: temp1 = 0b00000110;
		}
	switch (c)
		{
		case 0: temp2 = 0b11010000; break;
		case 1: temp2 = 0b01010000; break;
		case 2: temp2 = 0b01000000; break;
		case 3: temp2 = 0b00110000; break;
		case 4: temp2 = 0b00100000; break;
		case 5: temp2 = 0b10000000; break;
		case 6: temp2 = 0b10010000; break;
		case 7: temp2 = 0b00010000; break;
		case 8: temp2 = 0b00000000; break;
		case 9: temp2 = 0b11000000;
		}
	#asm("cli");
	byte1 = temp1+temp2;
	#asm("sei");
	switch (b)
		{
		case 0: temp1 = 0b00000100; break;
		case 1: temp1 = 0b00000000; break;
		case 2: temp1 = 0b00000010; break;
		case 3: temp1 = 0b00010010; break;
		case 4: temp1 = 0b00010000; break;
		case 5: temp1 = 0b00011000; break;
		case 6: temp1 = 0b00011010; break;
		case 7: temp1 = 0b00001010; break;
		case 8: temp1 = 0b00001000; break;
		case 9: temp1 = 0b00000110;
		}
	switch (d)
		{
		case 0: temp2 = 0b01000000; break;
		case 1: temp2 = 0b00000000; break;
		case 2: temp2 = 0b10000000; break;
		case 3: temp2 = 0b10000001; break;
		case 4: temp2 = 0b00000001; break;
		case 5: temp2 = 0b00100001; break;
		case 6: temp2 = 0b10100001; break;
		case 7: temp2 = 0b10100000; break;
		case 8: temp2 = 0b00100000; break;
		case 9: temp2 = 0b11000000;
		}
	#asm("cli");
	byte2 = temp1+temp2;
	#asm("sei");
	}

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer1_ovf_isr(void)	//   
	{
	TCNT0=0x64;
	
	#asm("wdr");
        
	volt = (read_adc(0)+read_adc(0)+read_adc(0)+read_adc(0))/4;
	if (volt >= 800 && OCR1AL>20) OCR1AL--;
	if (volt < 800 && OCR1AL<250) OCR1AL++;
	
	if (global_mode == clock_mode)
		{
		rtc_get_time();
		print_decode(time[0]/10, time[0]%10, time[1]/10, time[1]%10);
		hour = time[0];
		min = time[1];
		}
	
	SPCR.6 = 1;						 	//  SPI				
	SPDR = byte1; while ((SPSR & (1<<SPIF))==0) #asm ("wdr");   
      	SPDR = byte2; while ((SPSR & (1<<SPIF))==0) #asm ("wdr");   
      	strobe_toggle();                                                //  
      	SPCR.6 = 0;                                                     //  SPI
	}
	
// Timer 2 overflow interrupt service routine
interrupt [TIM2_OVF] void timer2_ovf_isr(void)	//    
	{
	TCNT2=0xB2;
	
	++button_cnt;
	if (button_cnt == 100)	{sound_on(50);};
	if (button_cnt == 200)	{sound_on(50); delay_ms(50); sound_on(50);};
	if (button_cnt == 300)	{sound_on(50); delay_ms(50); sound_on(50); delay_ms(50); sound_on(50);};
	}

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=Out Func4=In Func3=Out Func2=In Func1=Out Func0=Out 
// State7=T State6=T State5=0 State4=T State3=0 State2=T State1=1 State0=0 
PORTB=0x02;
DDRB=0x2B;

// Port C initialization
// Func6=In Func5=Out Func4=In Func3=In Func2=Out Func1=Out Func0=In 
// State6=T State5=0 State4=P State3=P State2=0 State1=0 State0=T 
PORTC=0x18;
DDRC=0x26;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=Out Func3=Out Func2=In Func1=Out Func0=In 
// State7=T State6=T State5=T State4=0 State3=0 State2=P State1=0 State0=P 
PORTD=0x05;
DDRD=0x1A;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 7,813 kHz
TCCR0=0x05;
TCNT0=0x64;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 8000,000 kHz
// Mode: Fast PWM top=00FFh
// OC1A output: Inverted
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0xC1;
TCCR1B=0x09;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0xB2;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// ADC initialization
// ADC Clock frequency: 1000,000 kHz
// ADC Voltage Reference: Int., cap. on AREF
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x83;

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/512k
#pragma optsize-
WDTCR=0x1D;
WDTCR=0x0D;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

SPIInit();

// I2C Bus initialization
i2c_init();

// DS1307 Real Time Clock initialization
// Square wave output on pin SQW/OUT: Off
// SQW/OUT pin state: 0
rtc_init(0,0,0);

i2c_start();		//   
i2c_write(0xd0);
i2c_write(0);
i2c_start();
i2c_write(0xd1);
null=i2c_read(0);
i2c_stop();
        
if (null&0b10000000)	//    
        {
        i2c_start();
        i2c_write(0xd0);
        i2c_write(0);
        i2c_write(null&0b01111111);	//  
        i2c_stop();
        }
        
if (a_hour > 23 || a_min > 59) a_hour = a_min = alarm_flag = 0;

OCR1AL=100;

#asm("sei");

while (1)
	{
	if (button && global_mode == clock_mode)	//        
		{
		delay_ms(5);                            // 
		if (button)                             //    
			{
			TIMSK=0x40;                     //    
			TCCR0=0x00;
			TCCR2=0x07;                     //   
 			SPCR.6 = 1;			//  
			SPDR = 0b00000000; while ((SPSR & (1<<SPIF))==0) #asm ("wdr");  
      			SPDR = 0b00101000; while ((SPSR & (1<<SPIF))==0) #asm ("wdr");   
      			strobe_toggle();
      			SPCR.6 = 0;
      			}
		while (button) #asm("wdr");		// ,   
		TCCR2=0x00;                             //     
		TCNT2=0xB2;
		if (button_cnt >= 300)	global_mode = clock_setup;		//      -  
			else if (button_cnt >= 200) global_mode = alarm_setup;  //      -  
				else if (button_cnt >= 100)                     //      - / 
					{
					alarm_flag = ~alarm_flag; 
					if (alarm_flag)			//    -  
						{
						led_r_on();             
						delay_ms(1000);
						}
					led_r_off();
					}
					else 	{                               //     -  
						global_mode = date_view;
						rtc_get_date();
						sound_on(50);
						TIMSK=0x01;		//     
						TCCR0=0x05;
						hour = date[0];
						min = date[1];
						print_decode(hour/10, hour%10, min/10, min%10);
						delay_ms(2000);
						hour = 20;
						min = date[2];
						print_decode(hour/10, hour%10, min/10, min%10);
						delay_ms(2000);
						rtc_get_time();
						global_mode = clock_mode;
						};
		button_cnt = 0;		//  
		
		TIMSK=0x01;		//     
		TCCR0=0x05;             
		delay_ms(20);
		}
		
	if (global_mode == clock_setup)	//     
		{
		led_b_on();
		#asm("cli");
		TCNT0=0x64;
		sec = 0;
		rtc_get_time();
		hour = time[0];
		min = time[1];
		print_decode(hour/10, hour%10, min/10, min%10);
		#asm("sei");
		while (!button) {#asm("wdr"); delay_ms(1); if (++timeout == 10000) {timeout = 0; goto exit1;};};	//  
		timeout = 0;
		while (button) {if (++hour == 24) hour = 0; print_decode(hour/10, hour%10, min/10, min%10); delay_ms(400);};
		time[0] = hour;
		sound_on(50);                                   // 
		while (!button) {#asm("wdr"); delay_ms(1); if (++timeout == 10000) {timeout = 0; goto exit1;};};	//  
		timeout = 0;
		while (button)	{if (++min == 60) min = 0; print_decode(hour/10, hour%10, min/10, min%10); delay_ms(400);};	//    -    
		time[1] = min;
		sound_on(50);                                   // 
		rtc_get_date();
		hour = 20;
		min = date[2];
		print_decode(hour/10, hour%10, min/10, min%10);
		while (!button) {#asm("wdr"); delay_ms(1); if (++timeout == 10000) {timeout = 0; goto exit1;};};	//  
		timeout = 0;
		while (button)	{if (++min == 100) min = 0; print_decode(hour/10, hour%10, min/10, min%10); delay_ms(400);};
		date[2] = min;
		sound_on(50);
		hour = date[0];
		min = date[1];
		print_decode(hour/10, hour%10, min/10, min%10);
		while (!button) {#asm("wdr"); delay_ms(1); if (++timeout == 10000) {timeout = 0; goto exit1;};};	//  
		timeout = 0;
		while (button)	{if (++min == 13) min = 1; print_decode(hour/10, hour%10, min/10, min%10); delay_ms(400);};
		date[1] = min;
		sound_on(50);
		while (!button) {#asm("wdr"); delay_ms(1); if (++timeout == 10000) {timeout = 0; goto exit1;};};	//  
		while (button)	
			{
			hour++;
			if ((hour > 28) && (date[1] == 2) &&  (date[2] % 4)) min = 1;                                          
                        if ((hour > 29) && (date[1] == 2) &&  !(date[2] % 4)) min = 1;                                         
                        if ((hour > 30) && ((date[1] == 4) || (date[1] == 6) || (date[1] == 9) || (date[1] == 11))) hour = 1;         
                        if (hour > 31) hour = 1;
			print_decode(hour/10, hour%10, min/10, min%10); 
			delay_ms(400);
			}
		date[0] = hour;
		sound_on(50);
		
 		#asm("cli");                                    //  
		i2c_stop();                                     //  ,      
		i2c_stop();
		i2c_stop();
		i2c_stop();                                     //  ,      
		i2c_stop();
		i2c_stop();
		rtc_set_time(time[0], time[1], 0);              //    RTC
		rtc_set_time(time[0], time[1], 0);              //    RTC
		rtc_set_time(time[0], time[1], 0);              //    RTC
		rtc_set_date(date[0], date[1], date[2]);        //    RTC
		rtc_set_date(date[0], date[1], date[2]);        //    RTC
		rtc_set_date(date[0], date[1], date[2]);        //    RTC
exit1:		#asm("cli");
		global_mode = clock_mode;                       //    
		#asm("sei");
		led_b_off(); led_r_off();
		}
		
	if (global_mode == alarm_setup)	//     
		{			//  - 
		led_g_on();
		#asm("cli");
		TCNT0=0x64;
		sec = 0;
		hour = a_hour;
		min = a_min;
		print_decode(hour/10, hour%10, min/10, min%10);
		#asm("sei");
		while (!button) {#asm("wdr"); delay_ms(1); if (++timeout == 10000) {timeout = 0; goto exit2;};};	//  
		timeout = 0;
		while (button) {if (++hour == 24) hour = 0; print_decode(hour/10, hour%10, min/10, min%10); delay_ms(400);};
		sound_on(50);
		while (!button) {#asm("wdr"); delay_ms(1); if (++timeout == 10000) {timeout = 0; goto exit2;};};	//  
		timeout = 0;
		while (button)	{if (++min == 60) min = 0; print_decode(hour/10, hour%10, min/10, min%10); delay_ms(400);};
		sound_on(50);
		#asm("cli");
		a_hour = hour;
		a_min = min;
exit2:		#asm("cli");
		hour = min = 0;
		global_mode = clock_mode;
		#asm("sei");
		led_g_off(); led_r_off();
		}
	
	if (hour != a_hour || min != a_min) alarm_complete = 0;	//      -   ,   
	if (global_mode == clock_mode && alarm_flag && hour == a_hour && min == a_min && alarm_complete == 0)	//         
		{
		alarm_complete = 1;
		TCCR0=0x00;		//  
		SPCR.6 = 1;		//  
		SPDR = 0b00000000; while ((SPSR & (1<<SPIF))==0) #asm ("wdr");  
      		SPDR = 0b00101000; while ((SPSR & (1<<SPIF))==0) #asm ("wdr");  
  		strobe_toggle();
  		SPCR.6 = 0;
		while (!button)				//   ,    
			{
			led_r_on();
			if (!button) sound_on(100);
			led_r_off();
			if (!button) delay_ms(100);
			led_r_on();
			if (!button) sound_on(100);
			led_r_off();
			if (!button) delay_ms(400);
			
			led_g_on();
			if (!button) sound_on(100);
			led_g_off();
			if (!button) delay_ms(100);
			led_g_on();
			if (!button) sound_on(100);
			led_g_off();
			if (!button) delay_ms(400);
			
			led_b_on();
			if (!button) sound_on(100);
			led_b_off();
			if (!button) delay_ms(100);
			led_b_on();
			if (!button) sound_on(100);
			led_b_off();
			if (!button) delay_ms(400);
			}
		while (button) #asm ("wdr");
		TCCR0=0x05;
		}
	};
}
